﻿using System;
using System.IO;
using System.Text;

namespace AdrianKousz.Util
{
	public class ExtendedBinaryReader : BinaryReader
	{
		public bool Strict;
		public Encoding Encoding { get; private set; }

		private const string ErrorChooseMethod = "Choose method with correct integer size";
		private const string ErrorHasNull = "String contains trailing null";
		private const string ErrorHasNoNull = "String does not contain trailing null";

		public ExtendedBinaryReader(Stream stream, Encoding enc, bool strict = true)
			: base(stream, enc)
		{
			Encoding = enc;
			Strict = strict;
		}

		public override string ReadString() { throw new NotSupportedException(ErrorChooseMethod); }
		public override bool ReadBoolean() { throw new NotSupportedException(ErrorChooseMethod); }

		public bool ReadBooleanByte() { return ReadByte() != 0; }
		public bool ReadBooleanInt16() { return ReadInt16() != 0; }
		public bool ReadBooleanInt32() { return ReadInt32() != 0; }

		public string ReadString(int len) { return ReadStringInternal(len, false); }
		public string ReadStringByte() { return ReadStringInternal(ReadByte(), false); }
		public string ReadStringInt16() { return ReadStringInternal(ReadInt16(), false); }
		public string ReadStringInt32() { return ReadStringInternal(ReadInt32(), false); }

		public string ReadZString(int len) { return ReadStringInternal(len, true); }
		public string ReadZStringByte() { return ReadStringInternal(ReadByte(), true); }
		public string ReadZStringInt16() { return ReadStringInternal(ReadInt16(), true); }
		public string ReadZStringInt32() { return ReadStringInternal(ReadInt32(), true); }

		private string ReadStringInternal(int len, bool expectNull)
		{
			if (len == 0) return "";
			var array = ReadBytes(len);
			var str = Encoding.GetString(array);

			var index = -1;
			if (expectNull || Strict) index = str.IndexOf("\0");
			if (Strict && !expectNull && index != -1) throw new FormatException(ErrorHasNull);
			if (Strict && expectNull && index == -1) throw new FormatException(ErrorHasNoNull);

			if (index == -1) return str;
			return str.Substring(0, index);
		}
	}
}
