﻿using System;
using System.IO;
using System.Text;

namespace AdrianKousz.Util
{
	public class ExtendedBinaryWriter : BinaryWriter
	{
		public Encoding Encoding { get; private set; }

		private const string ErrorChooseMethod = "Choose method with correct integer size";
		private const string ErrorStringTooLong = "String too long";

		public ExtendedBinaryWriter(Stream stream, Encoding enc)
			: base(stream, enc)
		{
			Encoding = enc;
		}

		public override void Write(string value) { throw new NotSupportedException(ErrorChooseMethod); }
		public override void Write(bool value) { throw new NotSupportedException(ErrorChooseMethod); }

		public void WriteBooleanByte(bool value) { if (value) Write((Byte)1); else Write((Byte)0); }
		public void WriteBooleanInt16(bool value) { if (value) Write((Int16)1); else Write((Int16)0); }
		public void WriteBooleanInt32(bool value) { if (value) Write((Int32)1); else Write((Int32)0); }

		public void WriteString(string value, int len) { WriteStringInternal(value, len, TypeCode.Empty); }
		public void WriteStringByte(string value) { WriteStringInternal(value, 0, TypeCode.Byte); }
		public void WriteStringInt16(string value) { WriteStringInternal(value, 0, TypeCode.Int16); }
		public void WriteStringInt32(string value) { WriteStringInternal(value, 0, TypeCode.Int32); }
		public void WriteStringRaw(string value) { WriteStringInternal(value, 0, TypeCode.Empty); }

		public void WriteZString(string value, int len) { WriteStringInternal(value + "\0", len, TypeCode.Empty); }
		public void WriteZStringByte(string value) { WriteStringInternal(value + "\0", 0, TypeCode.Byte); }
		public void WriteZStringInt16(string value) { WriteStringInternal(value + "\0", 0, TypeCode.Int16); }
		public void WriteZStringInt32(string value) { WriteStringInternal(value + "\0", 0, TypeCode.Int32); }
		public void WriteZStringRaw(string value) { WriteStringInternal(value + "\0", 0, TypeCode.Empty); }

		public int GetByteCount(string value) { return Encoding.GetByteCount(value); }
		public int GetZByteCount(string value) { return Encoding.GetByteCount(value + "\0"); }

		private void WriteStringInternal(string value, int len, TypeCode typecode)
		{
			byte[] array;
			if (len == 0) {
				array = Encoding.GetBytes(value);
				len = array.Length;
			} else {
				if (len < GetByteCount(value))
					throw new ArgumentOutOfRangeException(ErrorStringTooLong);
				array = new byte[len];
				Encoding.GetBytes(value, 0, value.Length, array, 0);
			}

			switch (typecode) {
				case TypeCode.Empty: break;
				case TypeCode.Int32: Write(checked((Int32)len)); break;
				case TypeCode.Int16: Write(checked((Int16)len)); break;
				case TypeCode.Byte: Write(checked((Byte)len)); break;
				default: throw new ArgumentException();
			}

			Write(array);
		}
	}
}
