﻿using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace AdrianKousz.Util
{
	public unsafe class UnsafeBitmap : IDisposable
	{
		private Bitmap bmp;
		private BitmapData bmpdat;

		public UnsafeBitmap(Bitmap bmp)
			: this(bmp, false) { }

		public UnsafeBitmap(Bitmap bmp, bool write)
		{
			this.bmp = bmp;
			var mode = write ? ImageLockMode.ReadWrite : ImageLockMode.ReadOnly;
			var rect = new Rectangle(0, 0, bmp.Width, bmp.Height);
			bmpdat = bmp.LockBits(rect, mode, bmp.PixelFormat);
		}

		public void Dispose()
		{
			if (bmp == null) return;
			bmp.UnlockBits(bmpdat);
			bmpdat = null;
			bmp = null;
		}

		public int Height { get { EnsureValid(); return bmpdat.Height; } }
		public PixelFormat PixelFormat { get { EnsureValid(); return bmpdat.PixelFormat; } }
		public byte* Scan0 { get { EnsureValid(); return (byte*)bmpdat.Scan0.ToPointer(); } }
		public int Stride { get { EnsureValid(); return bmpdat.Stride; } }
		public int Width { get { EnsureValid(); return bmpdat.Width; } }

		private void EnsureValid()
		{
			if (bmpdat == null)
				throw new ObjectDisposedException(null);
		}
	}
}

